/*
	ncbricks - ncurses based brick breaker
	Copyright (C) 2017-2018  Aron Barath

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
	02110-1301  USA
*/

#include "ncbricks.h"

static void update(struct paddle* const paddle)
{
	paddle->lrect.pos.x = LOGICAL_SCALE * paddle->x;
	paddle->lrect.pos.y = LOGICAL_SCALE * g_paddle_y;
	paddle->lrect.end.x = paddle->lrect.pos.x + (LOGICAL_SCALE * paddle->width) - 1;
	paddle->lrect.end.y = paddle->lrect.pos.y + LOGICAL_SCALE - 1;
}

void paddle_reset(struct paddle* const paddle)
{
	paddle->width = PADDLE_WIDTH;
	paddle->x = (g_board_width - PADDLE_WIDTH) / 2;
	update(paddle);
}

void paddle_show(const struct paddle* const paddle)
{
	int i;

	wmove(g_game_win, g_board_top+g_paddle_y, g_board_left+paddle->x);
	waddch(g_game_win, '<' | A_BOLD);
	for(i=2;i<paddle->width;++i) { waddch(g_game_win, '#' | A_BOLD); }
	waddch(g_game_win, '>' | A_BOLD);

	invalidate_board();
}

void paddle_hide(const struct paddle* const paddle)
{
	int i;
	char ch = ' ';
	if(g_backdef) { ch = '_'; }

	wmove(g_game_win, g_board_top+g_paddle_y, g_board_left+paddle->x);
	for(i=0;i<paddle->width;++i) { waddch(g_game_win, ch); }

	invalidate_board();
}

void paddle_move(struct paddle* const paddle, int amount)
{
	int x = paddle->x + amount;

	if(0>x)
	{
		x = 0;
	}
	else
	{
		const int max_x = g_board_width - paddle->width;

		if(max_x<=x)
		{
			x = max_x;
		}
	}

	if(x!=paddle->x)
	{
		paddle_hide(paddle);
		paddle->x = x;
		paddle_show(paddle);
		update(paddle);
	}
}

void paddle_enlarge(struct paddle* const paddle, int size)
{
	int max_x;

	if(0>size && PADDLE_MINW==paddle->width)
	{
		return;
	}
	else
	if(0<size && PADDLE_MAXW==paddle->width)
	{
		return;
	}

	paddle_hide(paddle);

	paddle->x     -= size;
	paddle->width += size*2;

	max_x = g_board_width - paddle->width;

	if(0>paddle->x)
	{
		paddle->x = 0;
	}
	else
	if(max_x<paddle->x)
	{
		paddle->x = max_x;
	}

	update(paddle);
	paddle_show(paddle);
}
