/*
	ncbricks - ncurses based brick breaker
	Copyright (C) 2017-2018  Aron Barath

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
	02110-1301  USA
*/

#include "ptrarr.h"
#include <stdlib.h>
#include <string.h>

#define DEFAULT_CAPACITY 16

void ptrarr_init(struct ptrarr* const arr)
{
	memset(arr, 0, sizeof(*arr));

	if(NULL==(arr->pointers=(void**)malloc(sizeof(void*)*DEFAULT_CAPACITY)))
	{
		abort();
	}

	arr->capacity = DEFAULT_CAPACITY;
}

void ptrarr_add(struct ptrarr* const arr, void* const ptr)
{
	if(arr->size==arr->capacity)
	{
		if(NULL==(arr->pointers=(void**)realloc(arr->pointers, 2*sizeof(void*)*arr->capacity)))
		{
			abort();
		}

		arr->capacity *= 2;
	}

	arr->pointers[arr->size++] = ptr;
}

void ptrarr_remove(struct ptrarr* const arr, size_t index)
{
	/* sanity check -- maybe a little bit too harsh, but who cares */
	if(index>=arr->size) { abort(); }

	if((index+1)<arr->size)
	{
		arr->pointers[index] = arr->pointers[arr->size-1];
	}

	--arr->size;
}
